@@ -547,7 +547,11 @@ ssize_t Read(int fd, void *buf, size_t count) {
    _errno = errno;
    if (!diag_in_handler) diag_flush();
 #if WITH_SYCLS
-   Debug1("read -> "F_Zd, result);
+   if (result < 0) {
+      Debug2("read -> "F_Zd" (errno=%d)", result, _errno);
+   } else {
+      Debug1("read -> "F_Zd, result);
+   }
 #endif /* WITH_SYCLS */
    errno = _errno;
    return result;
@@ -570,6 +574,7 @@ ssize_t Write(int fd, const void *buf, size_t count) {
    return result;
 }
 
+/* fcntl() without value */
 int Fcntl(int fd, int cmd) {
    int result, _errno;
    if (!diag_in_handler) diag_flush();
@@ -577,26 +582,47 @@ int Fcntl(int fd, int cmd) {
    Debug2("fcntl(%d, %d)", fd, cmd);
 #endif /* WITH_SYCLS */
    result = fcntl(fd, cmd);
-   if (!diag_in_handler) diag_flush();
+   if (!diag_in_handler)
+      diag_flush();
 #if WITH_SYCLS
    _errno = errno;
-   Debug1("fcntl() -> %d", result);
+   Debug1("fcntl() -> 0x%x", result);
    errno = _errno;
 #endif /* WITH_SYCLS */
    return result;
 }
 
+/* fcntl() with int value */
+int Fcntl_i(int fd, int cmd, int arg) {
+   int result, _errno;
+   if (!diag_in_handler) diag_flush();
+#if WITH_SYCLS
+   Debug3("fcntl(%d, %d, 0x%x)", fd, cmd, arg);
+#endif /* WITH_SYCLS */
+   result = fcntl(fd, cmd, arg);
+   _errno = errno;
+   if (!diag_in_handler)
+      diag_flush();
+#if WITH_SYCLS
+   Debug1("fcntl() -> 0x%x", result);
+#endif /* WITH_SYCLS */
+   errno = _errno;
+   return result;
+}
+
+/* fcntl() with long value */
 int Fcntl_l(int fd, int cmd, long arg) {
    int result, _errno;
    if (!diag_in_handler) diag_flush();
 #if WITH_SYCLS
-   Debug3("fcntl(%d, %d, %ld)", fd, cmd, arg);
+   Debug3("fcntl(%d, %d, 0x%lx)", fd, cmd, arg);
 #endif /* WITH_SYCLS */
    result = fcntl(fd, cmd, arg);
    _errno = errno;
-   if (!diag_in_handler) diag_flush();
+   if (!diag_in_handler)
+      diag_flush();
 #if WITH_SYCLS
-   Debug1("fcntl() -> %d", result);
+   Debug1("fcntl() -> 0x%x", result);
 #endif /* WITH_SYCLS */
    errno = _errno;
    return result;
@@ -663,31 +689,40 @@ int Flock(int fd, int operation) {
 }
 #endif /* HAVE_FLOCK */
 
-int Ioctl(int d, int request, void *argp) {
+/* Note that `request` is `int` in POSIX, but it is `unsigned long` for some
+   OSes e.g. Linux and OpenBSD.  This matters in some cases where truncation
+   would happen, so define it as `unsigned long` here. */
+int Ioctl(int d, unsigned long request, void *argp) {
    int retval, _errno;
    if (!diag_in_handler) diag_flush();
 #if WITH_SYCLS
    if (argp > (void *)0x10000) {	/* fuzzy...*/
-      Debug4("ioctl(%d, 0x%x, %p{%lu})", d, request, argp, *(unsigned long *)argp);
+      Debug7("ioctl(%d, 0x%lx, %p{0x%02x %02x %02x %02x})", d, request, argp, *(uint8_t *)argp, ((uint8_t *)argp)[1], ((uint8_t *)argp)[2], ((uint8_t *)argp)[3]);
    } else {
-      Debug3("ioctl(%d, 0x%x, 0x%p)", d, request, argp);
+      Debug3("ioctl(%d, 0x%lx, 0x%p)", d, request, argp);
    }
 #endif /* WITH_SYCLS */
    retval = ioctl(d, request, argp);
    _errno = errno;
    if (!diag_in_handler) diag_flush();
 #if WITH_SYCLS
-   Debug1("ioctl() -> %d", retval);
+   if (retval < 0)
+      Debug2("ioctl() -> %d, errno=%d", retval, _errno);
+   else
+      Debug1("ioctl() -> %d", retval);
 #endif /* WITH_SYCLS */
    errno = _errno;
    return retval;
 }
 
-int Ioctl_int(int d, int request, int arg) {
+/* Note that `request` is `int` in POSIX, but it is `unsigned long` for some
+   OSes e.g. Linux and OpenBSD.  This matters in some cases where truncation
+   would happen, so define it as `unsigned long` here. */
+int Ioctl_int(int d, unsigned long request, int arg) {
    int retval, _errno;
    if (!diag_in_handler) diag_flush();
 #if WITH_SYCLS
-   Debug3("ioctl(%d, 0x%x, %d)", d, request, arg);
+   Debug3("ioctl(%d, 0x%lx, %d)", d, request, arg);
 #endif /* WITH_SYCLS */
    retval = ioctl(d, request, arg);
    _errno = errno;
@@ -1094,13 +1129,13 @@ int Connect(int sockfd, const struct sockaddr *serv_addr, socklen_t addrlen) {
    Debug18("connect(%d,{0x%02x%02x%02x%02x %02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x}, "F_Zd")",
 	   sockfd,
 	   ((unsigned char *)serv_addr)[0],  ((unsigned char *)serv_addr)[1],
-	   ((unsigned char *)serv_addr)[2],  ((unsigned char *)serv_addr)[3], 
-	   ((unsigned char *)serv_addr)[4],  ((unsigned char *)serv_addr)[5], 
-	   ((unsigned char *)serv_addr)[6],  ((unsigned char *)serv_addr)[7], 
-	   ((unsigned char *)serv_addr)[8],  ((unsigned char *)serv_addr)[9], 
-	   ((unsigned char *)serv_addr)[10], ((unsigned char *)serv_addr)[11], 
-	   ((unsigned char *)serv_addr)[12], ((unsigned char *)serv_addr)[13], 
-	   ((unsigned char *)serv_addr)[14], ((unsigned char *)serv_addr)[15], 
+	   ((unsigned char *)serv_addr)[2],  ((unsigned char *)serv_addr)[3],
+	   ((unsigned char *)serv_addr)[4],  ((unsigned char *)serv_addr)[5],
+	   ((unsigned char *)serv_addr)[6],  ((unsigned char *)serv_addr)[7],
+	   ((unsigned char *)serv_addr)[8],  ((unsigned char *)serv_addr)[9],
+	   ((unsigned char *)serv_addr)[10], ((unsigned char *)serv_addr)[11],
+	   ((unsigned char *)serv_addr)[12], ((unsigned char *)serv_addr)[13],
+	   ((unsigned char *)serv_addr)[14], ((unsigned char *)serv_addr)[15],
 	   addrlen);
 #else
    Debug4("connect(%d, {%d,%s}, "F_socklen")",
@@ -1139,6 +1174,7 @@ int Listen(int s, int backlog) {
 #if _WITH_SOCKET
 /* don't forget to handle EINTR when using Accept() ! */
 int Accept(int s, struct sockaddr *addr, socklen_t *addrlen) {
+      char infobuff[256];
    int result, _errno;
    fd_set accept_s;
    if (!diag_in_handler) diag_flush();
@@ -1148,15 +1184,14 @@ int Accept(int s, struct sockaddr *addr, socklen_t *addrlen) {
       return -1;
    }
 #if WITH_SYCLS
-   Debug3("accept(%d, %p, %p)", s, addr, addrlen);
+   sockaddr_info(addr, *addrlen, infobuff, sizeof(infobuff));
+   Debug3("accept(%d, %p, %p)", s, infobuff, addrlen);
 #endif /* WITH_SYCLS */
    result = accept(s, addr, addrlen);
    _errno = errno;
    if (!diag_in_handler) diag_flush();
 #if WITH_SYCLS
    if (result >= 0) {
-      char infobuff[256];
-      sockaddr_info(addr, *addrlen, infobuff, sizeof(infobuff));
       Info5("accept(%d, {%d, %s}, "F_socklen") -> %d", s,
 	    addr->sa_family,
 	    sockaddr_info(addr, *addrlen, infobuff, sizeof(infobuff)),
@@ -1354,7 +1389,7 @@ int Sendto(int s, const void *mesg, size_t len, int flags,
 #if WITH_SYCLS
    sockaddr_info(to, tolen, infobuff, sizeof(infobuff));
    Debug7("sendto(%d, %p[%08x...], "F_Zu", %d, {%s}, %d)",
-	  s, mesg, htonl(*(unsigned long *)mesg), len, flags, infobuff, tolen);
+	  s, mesg, len>0?htonl(*(unsigned long *)mesg):0, len, flags, infobuff, tolen);
 #endif /* WITH_SYCLS */
    retval = sendto(s, mesg, len, flags, to, tolen);
    _errno = errno;
@@ -1392,11 +1427,11 @@ unsigned int Sleep(unsigned int seconds) {
 #if HAVE_NANOSLEEP
 unsigned int Nanosleep(const struct timespec *req, struct timespec *rem) {
    int retval, _errno;
-   Debug3("nanosleep({"F_time",%ld},%p)", req->tv_sec, req->tv_nsec, rem);
+   Debug3("nanosleep({"F_time".%09ld}, %p)", req->tv_sec, req->tv_nsec, rem);
    retval = nanosleep(req, rem);
    _errno = errno;
    if (rem) {
-      Debug3("nanosleep(,{"F_time",%ld}) -> %d",
+      Debug3("nanosleep(,{"F_time".%09ld}) -> %d",
 	     rem->tv_sec, rem->tv_nsec, retval);
    } else {
       Debug1("nanosleep() -> %d", retval);
@@ -1438,7 +1473,7 @@ struct hostent *Gethostbyname(const char *name) {
 int Getaddrinfo(const char *node, const char *service,
 		const struct addrinfo *hints, struct addrinfo **res) {
    int result;
-   Debug15("getaddrinfo(%s%s%s, %s%s%s, {%d,%d,%d,%d,"F_socklen",%p,%p,%p}, %p)",
+   Debug15("getaddrinfo(%s%s%s, %s%s%s, {0x%02x,%d,%d,%d,"F_socklen",%p,%p,%p}, %p)",
 	   node?"\"":"", node?node:"NULL", node?"\"":"",
 	   service?"\"":"", service?service:"NULL", service?"\"":"",
 	   hints->ai_flags, hints->ai_family, hints->ai_socktype,
@@ -1477,23 +1512,26 @@ struct hostent *Getipnodebyname(const char *name, int af, int flags,
 
 void *Malloc(size_t size) {
    void *result;
-   Debug1("malloc("F_Zd")", size);
+   Debug1("malloc("F_Zu")", size);
    result = malloc(size);
    Debug1("malloc() -> %p", result);
    if (result == NULL) {
-      Error1("malloc("F_Zd"): out of memory", size);
+      Error1("malloc("F_Zu"): out of memory", size);
       return NULL;
    }
+#if WITH_DEVTESTS
+   memset(result, 0x55, size);
+#endif /* WITH_DEVTESTS */
    return result;
 }
 
 void *Calloc(size_t nmemb, size_t size) {
    void *result;
-   Debug2("calloc("F_Zd", "F_Zd")", nmemb, size);
+   Debug2("calloc("F_Zu", "F_Zu")", nmemb, size);
    result = calloc(nmemb, size);
    Debug1("calloc() -> %p", result);
    if (result == NULL) {
-      Error2("calloc("F_Zd", "F_Zd"): out of memory", nmemb, size);
+      Error2("calloc("F_Zu", "F_Zu"): out of memory", nmemb, size);
       return NULL;
    }
    return result;
@@ -1501,16 +1539,28 @@ void *Calloc(size_t nmemb, size_t size) {
 
 void *Realloc(void *ptr, size_t size) {
    void *result;
-   Debug2("realloc(%p, "F_Zd")", ptr, size);
+   Debug2("realloc(%p, "F_Zu")", ptr, size);
    result = realloc(ptr, size);
    Debug1("realloc() -> %p", result);
    if (result == NULL) {
-      Error2("realloc(%p, "F_Zd"): out of memory", ptr, size);
+      Error2("realloc(%p, "F_Zu"): out of memory", ptr, size);
       return NULL;
    }
    return result;
 }
 
+/* Like Realloc(), but gets info about old size for overwrite test */
+void *Realloc3(void *ptr, size_t size, size_t oldsize) {
+   void *result = Realloc(ptr, size);
+   if (result == NULL)
+      return result;
+#if WITH_DEVTESTS
+   if (size > oldsize)
+      memset((char *)result+oldsize, 0x55, size-oldsize);
+#endif /* WITH_DEVTESTS */
+   return result;
+}
+
 #if _WITH_TERMIOS
 int Tcgetattr(int fd, struct termios *termios_p) {
    int i, result, _errno;
@@ -1533,7 +1583,7 @@ int Tcgetattr(int fd, struct termios *termios_p) {
 #else
    Debug6("tcgetattr(, {%08x,%08x,%08x,%08x,%s}) -> %d",
 	  termios_p->c_iflag, termios_p->c_oflag,
-	  termios_p->c_cflag, termios_p->c_lflag, 
+	  termios_p->c_cflag, termios_p->c_lflag,
 	  chars, result);
 #endif
    errno = _errno;
@@ -1736,6 +1786,28 @@ void Unsetenv(const char *name) {
 }
 #endif
 
+
+#if WITH_NAMESPACES
+int Setns(
+	int fd,
+	int nstype)
+{
+	int _errno;
+	int result;
+	Debug2("setns(%d, 0x%x)", fd, nstype);
+	result = setns(fd, nstype);
+	if (result < 0) {
+		_errno = errno;
+		Debug2("setns() -> %d (errno=%d)", result, errno);
+		errno = _errno;
+		return result;
+	}
+	Debug1("setns() -> %d", result);
+	return result;
+}
+#endif /* WITH_NAMESPACES */
+
+
 #if WITH_READLINE
 
 char *Readline(const char *prompt) {
